@@ -42,6 +42,7 @@
 #include "mime-types-model.h"
 
 static const gchar *get_category_name (const gchar        *mime_type);
+static const gchar *get_category_description (const gchar *mime_type);
 static GSList *get_lang_list          (void);
 static gchar  *form_extensions_string (const MimeTypeInfo *info,
 				       gchar              *sep,
@@ -51,8 +52,12 @@ static void    get_icon_pixbuf        (MimeTypeInfo       *info,
 				       gboolean            want_large);
 
 static MimeCategoryInfo *get_category (const gchar        *category_name,
+				       const gchar        *category_desc,
 				       GtkTreeModel       *model);
 
+gchar *get_real_icon_path             (const MimeTypeInfo *info,
+				       const gchar        *icon_name);
+
 
 
 void
@@ -79,7 +84,9 @@ mime_type_info_new (const gchar *mime_type, GtkTreeModel *model)
 	if (mime_type != NULL) {
 		info->mime_type = g_strdup (mime_type);
 
-		mime_type_info_set_category_name (info, get_category_name (mime_type), model);
+		mime_type_info_set_category_name (info, get_category_name (mime_type), get_category_description (mime_type), model);
+
+		mime_type_info_get_icon (info);
 	} else {
 		info->entry.parent = get_model_entries (model);
 	}
@@ -129,7 +136,8 @@ mime_type_info_get_icon (MimeTypeInfo *info)
 	if (info->small_icon_pixbuf == NULL)
 		get_icon_pixbuf (info, mime_type_info_get_icon_path (info), FALSE);
 
-	g_object_ref (G_OBJECT (info->small_icon_pixbuf));
+	if (info->small_icon_pixbuf != NULL)
+		g_object_ref (G_OBJECT (info->small_icon_pixbuf));
 
 	return info->small_icon_pixbuf;
 }
@@ -137,38 +145,10 @@ mime_type_info_get_icon (MimeTypeInfo *info)
 const gchar *
 mime_type_info_get_icon_path (MimeTypeInfo *info) 
 {
-	gchar *tmp;
-
 	if (info->icon_name == NULL)
 		info->icon_name = g_strdup (gnome_vfs_mime_get_icon (info->mime_type));
 
-	if (g_file_test (info->icon_name, G_FILE_TEST_EXISTS)) {
-		info->icon_path = g_strdup (info->icon_name);
-		return info->icon_path;
-	}
-
-	info->icon_path = gnome_vfs_icon_path_from_filename (info->icon_name);
-
-	if (info->icon_path == NULL) {
-		tmp = g_strconcat (info->icon_name, ".png", NULL);
-		info->icon_path = gnome_vfs_icon_path_from_filename (tmp);
-		g_free (tmp);
-	}
-
-	if (info->icon_path == NULL) {
-		tmp = g_strconcat ("nautilus/", info->icon_name, NULL);
-		info->icon_path = gnome_vfs_icon_path_from_filename (tmp);
-		g_free (tmp);
-	}
-
-	if (info->icon_path == NULL) {
-		tmp = g_strconcat ("nautilus/", info->icon_name, ".png", NULL);
-		info->icon_path = gnome_vfs_icon_path_from_filename (tmp);
-		g_free (tmp);
-	}
-
-	if (info->icon_path == NULL)
-		info->icon_path = gnome_vfs_icon_path_from_filename ("nautilus/i-regular-24.png");
+	info->icon_path = get_real_icon_path (info, info->icon_name);
 
 	return info->icon_path;
 }
@@ -212,7 +192,7 @@ mime_type_info_get_file_extensions_pretty_string (MimeTypeInfo *info)
 gchar *
 mime_type_info_get_category_name (const MimeTypeInfo *info)
 {
-	return mime_category_info_get_full_name (MIME_CATEGORY_INFO (info->entry.parent));
+	return mime_category_info_get_full_description (MIME_CATEGORY_INFO (info->entry.parent));
 }
 
 gboolean
@@ -221,7 +201,7 @@ mime_type_info_using_custom_app (const MimeTypeInfo *info)
 	gchar *tmp;
 	gboolean ret;
 
-	if (info->default_action->name == NULL)
+	if (!info->default_action || !info->default_action->name)
 		return TRUE;
 
 	tmp = g_strdup_printf ("Custom %s", info->mime_type);
@@ -232,13 +212,13 @@ mime_type_info_using_custom_app (const MimeTypeInfo *info)
 }
 
 void
-mime_type_info_set_category_name (const MimeTypeInfo *info, const gchar *category_name, GtkTreeModel *model) 
+mime_type_info_set_category_name (const MimeTypeInfo *info, const gchar *category_name, const gchar *category_desc, GtkTreeModel *model) 
 {
 	if (MODEL_ENTRY (info)->parent != NULL)
 		model_entry_remove_child (MODEL_ENTRY (info)->parent, MODEL_ENTRY (info), model);
 
 	if (category_name != NULL) {
-		MODEL_ENTRY (info)->parent = MODEL_ENTRY (get_category (category_name, model));
+		MODEL_ENTRY (info)->parent = MODEL_ENTRY (get_category (category_name, category_desc, model));
 
 		if (MODEL_ENTRY (info)->parent != NULL)
 			model_entry_insert_child (MODEL_ENTRY (info)->parent, MODEL_ENTRY (info), model);
@@ -327,7 +307,7 @@ mime_type_info_free (MimeTypeInfo *info)
 }
 
 MimeCategoryInfo *
-mime_category_info_new (MimeCategoryInfo *parent, const gchar *name, GtkTreeModel *model) 
+mime_category_info_new (MimeCategoryInfo *parent, const gchar *name, const gchar *description, GtkTreeModel *model) 
 {
 	MimeCategoryInfo      *info;
 
@@ -335,6 +315,7 @@ mime_category_info_new (MimeCategoryInfo *parent, const gchar *name, GtkTreeMode
 	MODEL_ENTRY (info)->type = MODEL_ENTRY_CATEGORY;
 
 	info->name = g_strdup (name);
+	info->description = g_strdup (description);
 
 	if (parent != NULL)
 		MODEL_ENTRY (info)->parent = MODEL_ENTRY (parent);
@@ -639,6 +620,25 @@ mime_category_info_get_full_name (MimeCategoryInfo *info)
 	return ret;
 }
 
+gchar *
+mime_category_info_get_full_description (MimeCategoryInfo *info) 
+{
+	GString *string;
+	ModelEntry *tmp;
+	gchar *ret;
+
+	string = g_string_new ("");
+
+	for (tmp = MODEL_ENTRY (info); tmp != NULL && tmp->type != MODEL_ENTRY_NONE; tmp = tmp->parent) {
+		g_string_prepend (string, MIME_CATEGORY_INFO (tmp)->description);
+		g_string_prepend (string, "/");
+	}
+
+	ret = g_strdup ((*string->str == '\0') ? string->str : string->str + 1);
+	g_string_free (string, TRUE);
+	return ret;
+}
+
 char *
 mime_type_get_pretty_name_for_server (Bonobo_ServerInfo *server)
 {
@@ -728,6 +728,25 @@ get_category_name (const gchar *mime_type)
 		return NULL;
 }
 
+static const gchar *
+get_category_description (const gchar *mime_type) 
+{
+	const gchar *path;
+
+	path = gnome_vfs_mime_get_value (mime_type, "category");
+
+	if (path != NULL)
+		return g_strdup (path);
+	else if (!strncmp (mime_type, "image/", strlen ("image/")))
+		return _("Images");
+	else if (!strncmp (mime_type, "video/", strlen ("video/")))
+		return _("Video");
+	else if (!strncmp (mime_type, "audio/", strlen ("audio/")))
+		return _("Audio");
+	else
+		return NULL;
+}
+
 static GSList *
 get_lang_list (void)
 {
@@ -774,6 +793,48 @@ form_extensions_string (const MimeTypeInfo *info, gchar *sep, gchar *prepend)
 	return tmp;
 }
 
+gchar *get_real_icon_path (const MimeTypeInfo *info, const gchar *icon_name) 
+{
+	gchar *tmp, *tmp1, *ret, *real_icon_name;
+
+	if (icon_name == NULL || *icon_name == '\0') {
+		tmp = g_strdup (info->mime_type);
+		tmp1 = strchr (tmp, '/');
+		if (tmp1 != NULL) *tmp1 = '-';
+		real_icon_name = g_strconcat ("document-icons/gnome-", tmp, ".png", NULL);
+		g_free (tmp);
+	} else {
+		real_icon_name = g_strdup (icon_name);
+	}
+
+	ret = gnome_vfs_icon_path_from_filename (real_icon_name);
+
+	if (ret == NULL && strstr (real_icon_name, ".png") == NULL) {
+		tmp = g_strconcat (real_icon_name, ".png", NULL);
+		ret = gnome_vfs_icon_path_from_filename (tmp);
+		g_free (tmp);
+	}
+
+	if (ret == NULL) {
+		tmp = g_strconcat ("nautilus/", real_icon_name, NULL);
+		ret = gnome_vfs_icon_path_from_filename (tmp);
+		g_free (tmp);
+	}
+
+	if (ret == NULL && strstr (real_icon_name, ".png") == NULL) {
+		tmp = g_strconcat ("nautilus/", real_icon_name, ".png", NULL);
+		ret = gnome_vfs_icon_path_from_filename (tmp);
+		g_free (tmp);
+	}
+
+	if (ret == NULL)
+		ret = gnome_vfs_icon_path_from_filename ("nautilus/i-regular-24.png");
+
+	g_free (real_icon_name);
+
+	return ret;
+}
+
 /* Loads a pixbuf for the icon, falling back on the default icon if
  * necessary
  */
@@ -784,7 +845,10 @@ get_icon_pixbuf (MimeTypeInfo *info, const gchar *icon_path, gboolean want_large
 	static GHashTable *icon_table = NULL;
 
 	if (icon_path == NULL)
-		icon_path = gnome_vfs_icon_path_from_filename ("nautilus/i-regular-24.png");
+		icon_path = get_real_icon_path (info, NULL);
+
+	if (icon_path == NULL)
+		return;
 
 	if ((want_large && info->icon_pixbuf != NULL) || info->small_icon_pixbuf != NULL)
 		return;
@@ -813,16 +877,25 @@ get_icon_pixbuf (MimeTypeInfo *info, const gchar *icon_path, gboolean want_large
 }
 
 static MimeCategoryInfo *
-get_category (const gchar *category_name, GtkTreeModel *model)
+get_category (const gchar *category_name, const gchar *category_desc, GtkTreeModel *model)
 {
 	ModelEntry *current, *child;
-	gchar **categories;
+	gchar **categories = NULL, **desc_categories = NULL;
 	int i;
 
-	if (category_name == NULL)
+	if (category_name == NULL && category_desc == NULL)
 		return NULL;
 
-	categories = g_strsplit (category_name, "/", -1);
+	if (category_name != NULL)
+		categories = g_strsplit (category_name, "/", -1);
+
+	if (category_desc != NULL)
+		desc_categories = g_strsplit (category_desc, "/", -1);
+
+	if (category_name == NULL)
+		categories = desc_categories;
+	else if (category_desc == NULL)
+		desc_categories = categories;
 
 	current = get_model_entries (model);
 
@@ -831,12 +904,16 @@ get_category (const gchar *category_name, GtkTreeModel *model)
 			if (child->type != MODEL_ENTRY_CATEGORY)
 				continue;
 
-			if (!strcmp (MIME_CATEGORY_INFO (child)->name, categories[i]))
+			if (!strcmp ((category_name == NULL) ?
+				     MIME_CATEGORY_INFO (child)->description :
+				     MIME_CATEGORY_INFO (child)->name,
+				     categories[i]))
 				break;
 		}
 
 		if (child == NULL) {
-			child = MODEL_ENTRY (mime_category_info_new (MIME_CATEGORY_INFO (current), categories[i], model));
+			child = MODEL_ENTRY (mime_category_info_new (MIME_CATEGORY_INFO (current), categories[i], 
+								     desc_categories[i], model));
 			model_entry_insert_child (MODEL_ENTRY (child)->parent, MODEL_ENTRY (child), model);
 		}
 
@@ -844,6 +921,7 @@ get_category (const gchar *category_name, GtkTreeModel *model)
 	}
 
 	g_strfreev (categories);
+	g_strfreev (desc_categories);
 
 	return MIME_CATEGORY_INFO (current);
 }
