@@ -148,12 +148,15 @@ static GConfValue *
 bell_to_widget (GConfPropertyEditor *peditor, const GConfValue *value)
 {
 	GConfValue *new_value;
+	const gchar *str;
 	gint val = 2;
 
+	str = (value && (value->type == GCONF_VALUE_STRING)) ? gconf_value_get_string (value) : NULL;
+
 	new_value = gconf_value_new (GCONF_VALUE_INT);
 	if (value->type == GCONF_VALUE_STRING) {
 		gconf_string_to_enum (bell_enums,
-				      gconf_value_get_string (value),
+				      str,
 				      &val);
 	}
 	gconf_value_set_int (new_value, val);
@@ -186,6 +189,17 @@ blink_to_widget (GConfPropertyEditor *peditor, const GConfValue *value)
 	return new_value;
 }
 
+static GConfValue *
+volume_to_widget (GConfPropertyEditor *peditor, const GConfValue *value)
+{
+	return gconf_value_int_to_float (value);
+}
+static GConfValue *
+volume_from_widget (GConfPropertyEditor *peditor, const GConfValue *value)
+{
+	return gconf_value_float_to_int (value);
+}
+
 static void
 bell_guard (GtkWidget *toggle,
 	    GladeXML  *dialog)
@@ -272,7 +286,10 @@ setup_dialog (GladeXML       *dialog,
 		(changeset, "/desktop/gnome/peripherals/keyboard/click", WID ("volume_toggle"), NULL);
 	gconf_peditor_widget_set_guard (GCONF_PROPERTY_EDITOR (peditor), WID ("volume_hbox"));
 	gconf_peditor_new_numeric_range
-		(changeset, "/desktop/gnome/peripherals/keyboard/click_volume", WID ("volume_scale"), NULL);
+		(changeset, "/desktop/gnome/peripherals/keyboard/click_volume", WID ("volume_scale"),
+		 "conv-to-widget-cb",   volume_to_widget,
+		 "conv-from-widget-cb", volume_from_widget,
+		 NULL);
 	
 	g_signal_connect (G_OBJECT (WID ("bell_custom_radio")), "toggled", (GCallback) bell_guard, dialog);
 	peditor = gconf_peditor_new_select_radio
@@ -308,7 +325,7 @@ static void
 setup_accessibility (GladeXML *dialog, GConfChangeSet *changeset)
 {
 	GtkWidget *notebook = WID ("notebook1");
-	GtkWidget *label = gtk_label_new (_("Accessibility"));
+	GtkWidget *label = gtk_label_new_with_mnemonic (_("_Accessibility"));
 	GtkWidget *page = setup_accessX_dialog (changeset, FALSE);
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
 }
